
#include "IceServer.hpp"
#include "Logging.hpp"
#include "matt.hpp"
#include <sstream>
#include <string>

namespace {

  ::std::string const appendIceDetails(
    Ice::Current const & c,
    ::std::string const & m) {
    ::std::ostringstream out;
    out << m << " - Requested by " << c.id.name;
    return out.str();
  }

  namespace detail {

    class IceMailbox : virtual public ::matt::messaging::Mailbox {

      bool isActive;

      std::string previous;

      bool empty(
        Ice::Current const &);

      bool active(
        Ice::Current const &);

      void deactivate(
        Ice::Current const &);

      std::string read(
        Ice::Current const &);

      void write(
        std::string const &, 
        Ice::Current const &);

      void addListener(
        matt::messaging::EventListenerPrx const &, 
        Ice::Current const &);

      void removeListener(
        Ice::Current const &);

    public:

      IceMailbox();
    };

    class IceMediator : virtual public ::matt::roles::Mediator {

    void IceMediator::registerParticipant(
      matt::roles::ParticipantPrx const &,
      Ice::Current const &);
    };
  }
}

detail::IceMailbox::IceMailbox() 
: isActive(true),
  previous() {
  MATT_DEBUG_LOG("Creating new IceMailbox");
}

bool detail::IceMailbox::empty(
  Ice::Current const & c) {
  MATT_DEBUG_LOG(appendIceDetails(
    c, "Checking if IceMailbox is empty")); 
  return previous.length() == 0;
}

bool detail::IceMailbox::active(
  Ice::Current const & c) {
  MATT_DEBUG_LOG(appendIceDetails(
    c, "Checking if IceMailbox is active"));
  return isActive;
}

void detail::IceMailbox::deactivate(
  Ice::Current const & c) {
  MATT_DEBUG_LOG(appendIceDetails(
    c, "Deactivating IceMailbox"));
  isActive = false;
}

std::string detail::IceMailbox::read(
  Ice::Current const & c) {
  std::string tmp;
  tmp.swap(previous);
  MATT_DEBUG_LOG(appendIceDetails(
    c, "reading \"" + tmp + "\" from IceMailbox"));
  return tmp;
}

void detail::IceMailbox::write(
  std::string const & m, 
  Ice::Current const & c) {
  MATT_DEBUG_LOG(appendIceDetails(
    c, "writing \"" + m + "\" to IceMailbox"));
  previous = m;
}

void detail::IceMailbox::addListener(
  matt::messaging::EventListenerPrx const &, 
  Ice::Current const &) {
  // NOT IMPLEMENTED 
}

void detail::IceMailbox::removeListener(
  Ice::Current const &) {
  // NOT IMPLEMENTED
}

void detail::IceMediator::registerParticipant(
  matt::roles::ParticipantPrx const & p,
  Ice::Current const & c) {

  MATT_DEBUG_LOG(appendIceDetails(
    c, "Registering new participant"));

  Ice::ObjectPrx proxy =  
    c.adapter->addWithUUID(new detail::IceMailbox());

  p->setMailbox(
    matt::messaging::MailboxPrx::checkedCast(proxy));

  MATT_DEBUG_LOG("Setting new participant's mailbox");
}

matt::IceServer::IceServer(
  Ice::CommunicatorPtr const & c)
: communicator(c) {

  MATT_DEBUG_LOG("Initialising IceServer");

  Ice::ObjectAdapterPtr adapter = 
    c->createObjectAdapter("matt.server");

  adapter->add(
    new detail::IceMediator(),
    c->stringToIdentity("matt.server"));

  adapter->activate();

  MATT_DEBUG_LOG("IceServer ready for requests");
}

void matt::IceServer::run() const {
  MATT_DEBUG_LOG("IceServer waiting for shutdown");
  communicator->waitForShutdown();
  MATT_DEBUG_LOG("IceServer shutting down");
}

