package matt.applications;

import org.apache.log4j.Logger;

import matt.adapters.MessageSource;
import matt.adapters.MessageTarget;
import matt.implementation.Controller;
import matt.implementation.DirectorySource;
import matt.implementation.EchoMessageSource;
import matt.implementation.EchoMessageTarget;
import matt.implementation.FileTarget;
import matt.implementation.Terminal;
import matt.protocol.Participant;
import matt.protocol.Runtime;

public final class Client extends Ice.Application {

  static private final Logger log = Logger.getLogger(Client.class);

  public static void main(
    String[] args) {
    final Client client = new Client();
    client.main("matt.client", args);
  }

  @Override
  public int run(
    String[] args) {
    final Runtime runtime = new Runtime(args, communicator());
    final Participant participant = new Participant();
    final MessageSource source = new DirectorySource(runtime);
    final MessageTarget target = new FileTarget(runtime);
    final MessageSource echoSource = new EchoMessageSource(source);
    final MessageTarget echoTarget = new EchoMessageTarget(target);
    final Controller relay = new Controller(participant, echoSource, echoTarget);
    
    //final MessageSource source = new Terminal();
    //final MessageTarget target = new Terminal();
    //final Controller relay = new Controller(participant, source, target);
    
    final Thread relayThread = new Thread(relay);
    runtime.register(participant);
    relayThread.start();
    runtime.waitForShutdown();
    echoTarget.dispose();
    echoSource.dispose();
    
    //target.dispose();
    //source.dispose();
    participant.dispose();    
    return 0;
  }
}
