package matt.implementation;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import matt.adapters.MessageTarget;
import matt.protocol.Runtime;

public class FileTarget
  implements
    MessageTarget {

  private final String filePath;
  private static final String MESSAGE_PREFIX = "message";
  private static final String EXCEPTION_PREFIX = "exception";
  private static final SimpleDateFormat FILE_SUFFIX = new SimpleDateFormat(
    "_yyyyMMddHHmmssSSS_");
  private static final DecimalFormat COUNTER_FORMAT = new DecimalFormat(
    "##000000");
  private final String fileExt;
  private int fileCount = 0;

  private File generateFile(
    final String prefix) {

    final Date now = new Date();
    final String filename = filePath + prefix + FILE_SUFFIX.format(now)
        + COUNTER_FORMAT.format(fileCount++) + fileExt;
    return new File(filename);
  }

  private void write(
    final File file,
    final String message) {
    try {
      final File tmp = File.createTempFile("Matt", "Temp");
      FileWriter writer = new FileWriter(tmp);
      writer.write(message);
      writer.flush();
      writer.close();
      tmp.renameTo(file);
    }
    catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
  }

  public FileTarget(
    final Runtime runtime) {

    final String path = runtime.getProperty("Matt.FileTarget.Path", "logs");
    final int seperator = path.indexOf(File.separatorChar);
    fileExt = "." + runtime.getProperty("Matt.FileTarget.Extension", "msg");
    if (seperator < path.length()) {
      filePath = path + File.separatorChar;
    }
    else {
      filePath = path;
    }
    File directory = new File(path);
    if (directory.exists()) {
      if (!directory.isDirectory()) {
        throw new RuntimeException(path
            + " already exists and is not a directory");
      }
    }
    else {
      directory.mkdirs();
    }
  }

  public void putMessage(
    String message) {
    write(generateFile(MESSAGE_PREFIX), message);
  }

  public void bubbleException(
    Exception e) {
    OutputStream stream = new ByteArrayOutputStream();
    PrintStream writer = new PrintStream(stream);
    e.printStackTrace(writer);
    write(generateFile(EXCEPTION_PREFIX), writer.toString());
  }

  public void dispose() {
    // TODO Auto-generated method stub
  }
  
  

}
