package matt.protocol;

import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import matt.messaging.MailboxPrx;
import matt.messaging.MailboxPrxHelper;
import matt.roles.MediatorPrx;
import matt.roles.MediatorPrxHelper;
import matt.roles.ParticipantPrx;
import matt.roles.ParticipantPrxHelper;
import Ice.Communicator;
import Ice.ObjectAdapter;

public class Runtime {

  static final String PARAMETER_REGEX = "--Matt\\.[\\w\\.]+=[\\W]*[^\\W]";
  static final Pattern PARAMETER_PARSER = Pattern.compile(PARAMETER_REGEX);

  private final Communicator communicator;
  private ObjectAdapter adapter = null; 
  private final Properties properties;

  public Runtime(
    final String[] args,
    final Communicator communicator) {
    this.communicator = communicator;
    properties = parseArgs(args);
  }

  static private final Properties parseArgs(
    final String[] args) {
    final Properties properties = new Properties();
    for (final String arg : args) {
      final Matcher raw_pair = PARAMETER_PARSER.matcher(arg);
      if (raw_pair.find()) {
        final int name_start = raw_pair.start() + 2; // skip the "--"
        final int equal_start = arg.indexOf('=');
        final String name = arg.substring(name_start, equal_start);
        final String value = arg.substring(equal_start + 1);
        properties.setProperty(name, value);
      }
    }
    return properties;
  }

  private final ParticipantPrx createParticipantPrx(
    final Participant client) {

    adapter = communicator.createObjectAdapter("matt.client");
    adapter.activate();

    return ParticipantPrxHelper.checkedCast(adapter.addWithUUID(client
      .getParticipant()));
  }

  private final MediatorPrx createMediatorPrx(
    final Mediator server) {

    adapter = communicator.createObjectAdapter("matt.server");
    adapter.activate();

    return MediatorPrxHelper.checkedCast(adapter.add(
      server.getMediator(),
      communicator.stringToIdentity("matt.server")));
  }

  private final MediatorPrx createMediatorPrx() {
    return MediatorPrxHelper.checkedCast(communicator
      .propertyToProxy("matt.server"));
  }
  
  final MailboxPrx createMailboxPrx(final matt.messaging.Mailbox mailbox) {
    return MailboxPrxHelper.checkedCast(adapter.addWithUUID(mailbox));
  }

  public void register(
    final Mediator server) {
    createMediatorPrx(server);
  }

  public void register(
    final Participant client) {
    ParticipantPrx participant = createParticipantPrx(client);
    MediatorPrx mediator = createMediatorPrx();
    mediator.registerParticipant(participant);
  }

  public String getProperty(
    final String name) {
    return properties.getProperty(name);
  }

  public String getProperty(
    final String name,
    final String defaultValue) {
    final String value = properties.getProperty(name, defaultValue);
    System.out.println(name + " = " + value + " (" + defaultValue + ")");
    return value;
  }
  
  public void waitForShutdown() {
    communicator.waitForShutdown();
  }
}
